@@ -5,6 +5,7 @@ use std::{
     collections::VecDeque,
     io::{Read, Write},
     ops::{Deref, DerefMut},
+    mem,
 };
 
 use bincode;
@@ -77,11 +78,7 @@ impl AdhocCodec {
         self
     }
 
-    pub fn info(&self) -> StreamInfo {
-        self.stream
-            .info()
-            .expect("info not initalized, call set_info/with_info before decoding")
-    }
+
 
     pub fn set_info(&mut self, info: StreamInfo) {
         let num_channels = info.channels as usize;
@@ -90,6 +87,41 @@ impl AdhocCodec {
         self.stream.set_info(Some(info));
     }
 
+    fn info(&self) -> StreamInfo {
+        self.stream
+            .info()
+            .expect("info not initalized, call set_info/with_info before decoding")
+    }
+
+
+    /// # Description 
+    /// returns a tight upper-bound estimate on the number of bits needed to write this file 
+    fn filesize_upperbound(&self)->u64{
+        /*
+            the final filesize is roughly equal to:
+                FrameHeaderList_bits+
+                AudioStream_bits + 
+                compression_level_bits 
+        */
+        let audio_stream_header =         
+            //number of bits AudioStream need to store info 
+            mem::size_of::<StreamInfo>() as u64 * 8; 
+
+        let bitstream = self.stream.capacity_upperbound() as u64+ 
+          //streams internal cursor + capacity 
+          128*2 ;
+
+
+        let audio_stream_bits = audio_stream_header + bitstream;
+        let compression_level_bits = 32;
+        let frame_header_bits =  self.frame_header_list.calculate_weight_upperbound();
+
+        //estimated upperbound is..
+        audio_stream_bits + 
+        compression_level_bits + 
+        frame_header_bits
+    }
+
     /// # Description
     /// use this to specify compression `level` where level is 0-10, where 0 is little to no loss in quality while 10 is very,very lossy
     pub fn with_compression_level(mut self, mut level: u32) -> Self {
@@ -258,6 +290,15 @@ impl AdhocCodec {
 }
 
 impl Streamable for AdhocCodec {
+    
+    fn info(&self) -> StreamInfo {
+        self.info()
+    }
+
+    fn filesize_upperbound(&self) ->u64 {
+        self.filesize_upperbound()
+    }
+
     fn encode(&mut self, samples: &[f32]) -> Option<usize> {
         self.encode(samples);
 
@@ -275,6 +316,14 @@ impl Streamable for AdhocCodec {
 
         self.init();
 
+        if let SeekFrom::Start(0) = dt{
+            // this will skip the process of decoding the frame 
+            // so you can call encode(..) if you want to reuse data that has been 
+            // previously allocated 
+            return; 
+        }
+
+
         let frame_header_list = &mut self.frame_header_list;
         let channel_state_list = &mut self.channel_state_list;
         let stream = &mut self.stream;
@@ -403,7 +452,7 @@ mod test {
         println!("decoded:{:?}", &out_buffer[0..samples_read]);
         println!(
             "blocks allocated = {}\nbits written = {}\ncompression_ratio={}\nMean Squared Error={:.6}",
-            codec.stream.binary.len(),
+            codec.stream.blocks_allocated(),
             codec.stream.capacity(),
             codec.stream.capacity() as f32 / (data.len() * 16) as f32,
             mean_squared_error
@@ -439,7 +488,7 @@ mod test {
         println!("decoded:{:?}", &out_buffer[0..samples_read]);
         println!(
             "blocks allocated = {}\nbits written = {}\ncompression_ratio={}\nMean Squared Error={:.6}",
-            codec.stream.binary.len(),
+            codec.stream.blocks_allocated(),
             codec.stream.capacity(),
             codec.stream.capacity() as f32 / (data.len() * 16) as f32,
             mean_squared_error
